import wishbone_pkg::*;

`include "vhd_wishbone_master.svh"
`include "cpu_csr_driver.svh"
`include "mqueue_host.svh"

`timescale 1ns/1ps

module main;

   reg rst_n = 0;
   reg clk_sys = 0;
   reg clk_cpu = 0;

   always #4ns clk_cpu <= ~clk_cpu;
   always@(posedge clk_cpu)
     clk_sys <= ~clk_sys;


   initial begin
      repeat(20) @(posedge clk_sys);
      rst_n = 1;
   end

   wire host_irq;

   IVHDWishboneMaster Host ( clk_sys, rst_n );

   mock_turtle_core #
     (
      .g_double_core_clock(1'b0),
      //.g_cpu_arch("LM32")
      .g_cpu_arch("URV")
      )
   DUT (
	.clk_i   (clk_sys),
	.clk_cpu_i(clk_cpu),
	.rst_n_i (rst_n),
	.host_slave_i (Host.master.out),
	.host_slave_o (Host.master.in),
	.host_irq_o(host_irq)
	);

   initial begin
      NodeCPUControl cpu_csr;
      MQueueHost hmq;
      uint64_t rv;
      CBusAccessor host_acc;


      #10us;


      host_acc = Host.get_accessor();


      cpu_csr = new ( Host.get_accessor(), 'hc000 );
      hmq = new ( Host.get_accessor(), 0 );

      // enable all IRQs
      host_acc.write(`MQUEUE_GCR_IRQ_MASK, 'hffff);

      cpu_csr.init();

      //cpu_csr.load_firmware (0, "../sw/hw-tests/hello/hello.ram.lm32");
      cpu_csr.load_firmware (0, "../sw/hw-tests/hello/hello.ram.urv");
      cpu_csr.reset_core(0, 0);

      $display("CPU0 started\n");

      forever begin
	 cpu_csr.update();
	 while(host_irq)
	   hmq.update();
	 #1us;
	 @(posedge clk_sys);
      end

   end // initial begin

endmodule
